#!/usr/bin/perl -w
#
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://public.eu-egee.org/partners/ for details on 
# the copyright holders.
# For license conditions see the license file or
# http://eu-egee.org/license.html
#

use English;
use Getopt::Long;
Getopt::Long::Configure('bundling');

sub transfer;
sub post_transfer;
sub stop_test;
sub direction1;

my $home = $ENV{'HOME'};
my $data_dir;
my $cycle_number;
my $config_file;

GetOptions (    "conffile|f=s" => \$config_file,
                "dir|d=s" => \$data_dir,
		"cycle|c=s" => \$cycle_number,
                );

if (scalar(@ARGV) != 2) {
        print "ARGV:\n";
        for $i (@ARGV) {
                print "$i\n";
        }
        print "\nusage: $0 <file index>\n\n";
        stop_test("Bad command line: $0");
}

$data_dir =~ s/\/$//;
my $file_index = $ARGV[0];
my $run_id = $ARGV[1];
my $orig = $data_dir . "/orig";
my $copy = $data_dir . "/copy${run_id}-$file_index";
my $lfncopy = "copy${run_id}-${cycle_number}-$file_index";
my $failure = "$data_dir/failure";
my $time_command = '/usr/bin/time';
my $cmd;

(-e $copy) and system("rm -f $copy") and stop_test("Failed removing $copy.");

print "\nFile Index = " . $file_index . "\n";
print "Run_id = " . $run_id . "\n\n";
print "\n-------------- $cycle_number -------------- \n\n";

# print command line
my $cmdline = `cat /proc/$PID/cmdline`;
$cmdline =~ s/\x0/ /g;
print "COMMAND:  $cmdline\n";

direction1;

sub direction1 {
	$cmd = "$time_command glite-put -c $config_file $orig lfn:///$lfncopy";
	transfer;
	$cmd = "$time_command glite-get -c $config_file lfn:///$lfncopy $copy";
	transfer;
	post_transfer;
}

sub post_transfer {
	# compare copy with original
	system("diff -q $orig $copy") and stop_test("diff: Copy $copy is corrupted."); 
 
	# remove copies
	my $host = `hostname -s`;
	system("rm -f $copy") and stop_test("Failed removing $host:$copy.");
}

sub transfer {
	my $ltime = localtime;
	print "START $ltime\nCOMMAND: $cmd\n";
	system($cmd) and stop_test("$cmd    failed.");
	$ltime = localtime;
	print "END $ltime\n\n";
}

sub stop_test {
	my $msg = shift;
	my $ltime = localtime;
	my $str = "$file_index ";  
	print ("\nERROR: $msg\n");
	system("/bin/echo -n $str\n");
	system("/bin/echo -n $str >> $failure") and print "\nERROR: Failed writing to $failure.\n";
# moving the io-client log to $data_dir
       system("mv $home/.glite-io-client.log $data_dir/run$run_id") and print "\nFailed moving io-client log file to $data_dir.\n";

	exit 1;
}

