#!/usr/bin/perl
                                                                                                                                     
use POSIX qw(sys_wait_h log10);
use IO::Handle;
STDOUT->autoflush(1);
use Getopt::Long;
Getopt::Long::Configure('bundling');
                                                                                                                                     
sub fork_exec;
sub set_command;
sub write_head_log;

my $HOME 	    = $ENV{'HOME'};
my $BIN_PATH 	= "/opt/glite/bin";
my $GLITE_PUT   = "$BIN_PATH/glite_put -v";
my $GLITE_GET   = "$BIN_PATH/glite_get -v";
my $GLITE_RM    = "$BIN_PATH/glite_rm  -v";
my $LFN_PREFIX  = "lfn:///badino_test";

# Config Variables
my $g_maxFileSize     = 10**7;
my $g_numberFiles     = 1;
my $g_maxFileSizeSum  = 10**8;
my $g_numberCycles    = 10;
my $g_numberIteration = 100;
my $g_cfgName         = "glite-io-client-egtest-oracle";

my $g_logDir;

# Report Variables
my $g_uploadSuccess    = 0;
my $g_uploadFailures   = 0;
my $g_downloadSuccess  = 0;
my $g_downloadFailures = 0;
my $g_unlinkSuccess    = 0;
my $g_unlinkFailures   = 0;

#
# Print Report
#
sub print_reports {
    my $cycle = shift;
    my $num_uploads   = $g_uploadSuccess    + $g_uploadFailures;
    my $num_downloads = $g_downloadSuccess  + $g_downloadFailures;
    my $num_unlinks   = $g_unlinkSuccess    + $g_unlinkFailures;
    my $up_failures   = 0;
    my $down_failures = 0;
    my $rm_failures   = 0;
    if(0 != $num_uploads) {
	$up_failures = $g_uploadFailures   / $num_uploads * 100;
    }
    if(0 != $num_downloads) {
        $down_failures = $g_downloadFailures / $num_downloads * 100;
    }
    if(0 != $num_unlinks) {
        $rm_failures   = $g_unlinkFailures   / $num_unlinks * 100;
    }
    my $cycles_num    = $cycle / $g_numberCycles * 100;

    print "************************************************************************\n";
    print "* Executed $cycle Cycles of $g_numberCycles ($cycles_num %)\n";
    print "*   Uploads  : $num_uploads  - Failures $g_uploadFailures ($up_failures %)\n";
    print "*   Downloads: $num_downloads  - Failures $g_downloadFailures ($down_failures %)\n";
    print "*   Unlinks  : $num_unlinks  - Failures $g_unlinkFailures ($rm_failures %)\n";
    print "************************************************************************\n";

    if(($up_failures > 50) || ($down_failures > 50)) {
       return -1;
    } else {
      return 0;
    }

}
#
# Upload File
#
sub upload_file {
    my $source = shift;
    my $lfn    = shift;
    my $log_filename = shift;
    
    my $cmd_line = "$GLITE_PUT $source $lfn"; 
    
    my $result = exec_cmd($cmd_line, $log_filename);
    if(0 == $result){
        $g_uploadSuccess++;
    } else {
        $g_uploadFailures++;
    }

    return $result;
}

#
# download File
#
sub download_file {
    my $lfn   = shift;
    my $dest  = shift;
    my $log_filename = shift;
    
    my $cmd_line = "$GLITE_GET $lfn $dest"; 
    
    my $result = exec_cmd($cmd_line, $log_filename);
    if(0 == $result){
        $g_downloadSuccess++;
    } else {
        $g_downloadFailures++;
    }
    
    return $result;
}

#
# unlink File
#
sub unlink_file {
    my $lfn    = shift;
    my $log_filename = shift;
                                                                                                             
    my $cmd_line = "$GLITE_RM $lfn";
                                                                                                             
    my $result = exec_cmd($cmd_line, $log_filename);
    if(0 == $result){
        $g_unlinkSuccess++;
    } else {
        $g_unlinkFailures++;
    }
                                                                                                             
    return $result;
}


#
# Compare Files
#
sub compare_files {
    my $source = shift;
    my $dest   = shift;
    my $log_filename = shift;
    
    my $cmd_line = "diff -q $source $dest"; 
    
    my $result = exec_cmd($cmd_line, $log_filename);
    if(0 != $result){ 
        print "diff: Copy $dest is corrupted\n";
    }                                            

    return $result;
}

#
# Remove file
#
sub remove_file {
    my $fname = shift;
    system("rm -f $fname");
}

#
# Crate a Directory for logging
#
sub get_log_dir {
    my $run_id         = shift;
    my $run_dir        = $g_logDir . "/run$run_id";
    my $logs_file_base = $run_dir  . "/log";

    mkdir($run_dir       , 0755) or die "Can't mkdir $run_dir.\n";
    mkdir($logs_file_base, 0755) or die "Can't mkdir $logs_file_base.\n";
    
    return $logs_file_base;
}

#
# Execute a command and wait the result
#
sub exec_cmd {
    
    my $result  = 0;
    my $TIME_CMD = "/usr/bin/time";

    my $cmd          = $TIME_CMD . " " . shift;
    my $log_filename = shift;
    
    my $ltime = localtime;
    system("echo \"START $ltime\nCOMMAND: $cmd\n\" >> $log_filename 2>&1");

    #fork Child process
    my $pid=fork();
    if (!defined($pid)) {
        die "Cannot fork: $!";
    } elsif ($pid) { # Parent
        # Wait the Child Process finished
        waitpid($pid,0);
        $result = $?
    } else { # Child
        exec("$cmd >> $log_filename 2>&1");
    }

    return $result;
}
#
# Translate Size String
#
sub get_size {
    
    my $size_str = shift;
    my $size     = shift;
    if ( $size_str ) {
        if ( $size_str =~ /^(\d+)([kmgKMG])/ ) {
            my $unit = $2;
            $size = $1;
            my $k = $unit =~ /^[kK]/ ? 1024 : $unit =~ /^[mM]/ ? 1024**2 : 1024**3;
            $size *= $k;
        } else {
            $size = eval($size_str);
#            print "Bad size: $size_str\n";
#            exit 1;
        }
    }
    return $size;
}

#
# get_options
#
sub get_options{

	my $max_file_size_string;
	my $max_file_size_sum_string;

	GetOptions ("mfs|f=s"  => \$max_file_size_string,
    	        "mnf|n=s"      => \$g_numberFiles,
                "mfss|s=s"     => \$max_file_size_sum_string,
                "cycles|c=s"   => \$g_numberCycles,
                "iter|i=s"     => \$g_numberIteration,
                "log|l=s"      => \$g_logDir,
                "cfg|g=s"      => \$g_cfgName,
                );
	unless ($g_logDir ) {
        print_usage();
        exit 1;
	}
    # set Max File Size
    $g_maxFileSize = get_size($max_file_size_string, $g_maxFileSize);

    # set Max File Size Sum
    $g_maxFileSizeSum = get_size($max_file_size_sum_string, $g_maxFileSizeSum);
    
    if( $g_cfgName ) {
        $GLITE_PUT = "$GLITE_PUT -s $g_cfgName ";
        $GLITE_GET = "$GLITE_GET -s $g_cfgName ";
        $GLITE_RM  = "$GLITE_RM  -s $g_cfgName ";
    }
}
                                                                                                                                     
#
# Generate Random File
#
sub generate_file {
    
    my $file_name       = shift;
    my $random_source   = "/dev/urandom";
    
    # generate file size
    my $file_size       = int(10**rand(log10($g_maxFileSize))) - 1;
    my $max_record_size = 1000000;
    
    # generate file
    print " => Creating random file. Size: $file_size.\n";
    open(RAND, "< $random_source") or die "Couldn't open $random_source for reading: $!\n";
    open(ORIG, "> $file_name")  or die "Couldn't open $file_name for writing: $!\n";
    my $iterations = int($file_size/$max_record_size);
    my $record;

    for (my $i = 0; $i < $iterations; $i++) {
        # Read Record from /dev/random
        read(RAND, $record, $max_record_size) == $max_record_size or 
            die "Failed reading $max_record_size bytes from $random_source.\n";
        # Write Record To File
        syswrite(ORIG, $record, $max_record_size) == $max_record_size or 
            die "Failed writing  $max_record_size bytes to $orig.\n";
    }
    
    #write the end of the file                                                                                                                                 
    my $rest = $file_size - $iterations * $max_record_size;
    read(RAND, $record, $rest) == $rest  or die "Failed reading $rest bytes from $random_source.\n";
    syswrite(ORIG, $record, $rest) == $rest or die "Failed writing $rest bytes to $orig.\n";
    
    # Close The Files
    close(RAND);
    close(ORIG);
}

#
# Print Program Usage
#
sub print_usage {
    print <<"EOF";
The script runs a series of get/put tests of the gLite IOserver.
    
Usage:
    TODO                                                                                                                                     
EOF
}

################################################################################
# Main
################################################################################

# Get Options
get_options();
    
my $run_id = time;

my $log_dir = get_log_dir($run_id);

# Do Iteration
my $cycle = 0;
my $iter  = 0;
for($cycle = 0; $cycle < $g_numberCycles;  $cycle++) {
    my $orig        = $g_logDir . "/orig";
    my $dest        = $g_logDir . "/dest";
    
    my $fault_count = 0;
    my $file_index  = 0;
    
    my $lfn          = "$LFN_PREFIX-${run_id}-${cycle}-$file_index";
    my $log_filename = "$log_dir/$cycle";
   
    print "Executing cycle " . int($cycle+1) . " of $g_numberCycles\n => Log file is $log_filename\n";
    
    # Generate random File
    generate_file($orig);

    # Upload File
    my $result = upload_file($orig, $lfn, $log_filename);
    if(0 == $result){
        # reset fault count
        $fault_count = 0;
        
        # do downloads
        for($iter = 0; $iter < $g_numberIteration; $iter++){
            
            print " => Iteration " . int($iter+1) . " of $g_numberIteration\r";

            # Download File
            $result = download_file($lfn, $dest, $log_filename);
            if(0 == $result){
                # reset fault count
                $fault_count = 0;
    
                # Compare Files
                $result = compare_files($orig,$dest, $log_filename);
                if(0 != $result){
                    $fault_count++;
                }
                
                # Remove File
                remove_file($dest)
            } else {
                $fault_count++;
                if($fault_count > 10){
                    die "Too many consecutive faults";
                }
            }
        }

	# Unlink the file
	$result = unlink_file($lfn, $log_filename);
	if(0 != $result){
		$fault_count++;
        	if($fault_count > 10){
            		die "Too many consecutive faults";
        	}
	}
    } else {
        $fault_count++;
        if($fault_count > 10){
            die "Too many consecutive faults";
        }
    }

    # Remove Source File        
    remove_file($orig);

    if($fault_count > 0){ 
        print " => There are faults !!!\n";
    }
} # End for

# Print Reports
print_reports($cycle);
